#include<string>
#include<iostream>
using namespace std;

//Superclass of all kinds of objects to be created
//Generically called Product
class Pizza
{
  public:
    //Set all parts of the Product, and use the Product
    void setDough(const string& dough){
       m_dough = dough;}
    void setSauce(const string& sauce){
       m_sauce = sauce;}
    void setTopping(const string& topping){
       m_topping = topping;}
    void open() const{
       cout << "Pizza with " << m_dough 
            << " dough, " << m_sauce << " sauce and "
            << m_topping << " topping. Mmm." << endl;}
  private:
    //Parts of the Product
    string m_dough;
    string m_sauce;
    string m_topping;
};

class PizzaBuilder
{
  public:
    //get the built Pizza from the Builder
    Pizza* getPizza(){ return m_pizza; }
    //build a generic empty Pizza
    void createNewPizzaProduct(){ m_pizza = new Pizza; }
    //create each part of the Product according to subtypes
    virtual void buildDough() = 0;
    virtual void buildSauce() = 0;
    virtual void buildTopping() = 0;
  protected:
    //Product built by Pizza Builder
    Pizza* m_pizza;
};

class HawaiianPizzaBuilder : public PizzaBuilder //Concrete Builder 1
{
  public:
    virtual void buildDough()                    //Build different parts of the pizza 
      {m_pizza->setDough("cross");}              //The construction process could be
    virtual void buildSauce()                    //more complex in a real-life example
      {m_pizza->setSauce("mild");}               //The construction of the pizza part 
    virtual void buildTopping()                  //depends on the type of pizza. 
      {m_pizza->setTopping("ham+pineapple");}
};
 
class SpicyPizzaBuilder : public PizzaBuilder    //Concrete Builder 2
{
  public:
    virtual void buildDough()                    //The construction process may vary 
      {m_pizza->setDough("pan baked");}          //across different Concrete Builders. 
    virtual void buildSauce()
      {m_pizza->setSauce("hot");}
    virtual void buildTopping()
      {m_pizza->setTopping("pepperoni+salami");}
};

class Cook //Director
{
  public:
    void setPizzaBuilder(PizzaBuilder* pb) //Use a concrete builder
      {                                    //for building a specific
         m_pizzaBuilder = pb;              //kind of Pizza.
      }
    Pizza* getPizza()                      //get the constructed Pizza
      {
         return m_pizzaBuilder->getPizza();
      }
    void constructPizza()                  //Creational process to create 
      {                                    //a pizza using the builder. 
         m_pizzaBuilder->createNewPizzaProduct(); 
         m_pizzaBuilder->buildDough();
         m_pizzaBuilder->buildSauce();
         m_pizzaBuilder->buildTopping();
      }
  private:
    PizzaBuilder* m_pizzaBuilder;
};

//Pizza Builder Client
int main() 
{
        Cook cook;				//Create the Director 
        PizzaBuilder* hawaiianPizzaBuilder = 	//Create the Concrete Builder
	new HawaiianPizzaBuilder;
        PizzaBuilder* spicyPizzaBuilder = 
	new SpicyPizzaBuilder;
 
        cook.setPizzaBuilder(hawaiianPizzaBuilder);   //Tell the Director which Builder to use
        cook.constructPizza();                        //Tell the Director to construct the Product
        Pizza* hawaiian = cook.getPizza();            //Client gets the Product
        hawaiian->open();                             //Client uses the Product
 
        cook.setPizzaBuilder(spicyPizzaBuilder);      //same for another kind of product
        cook.constructPizza();
 
        Pizza* spicy = cook.getPizza();
        spicy->open();
 
        delete hawaiianPizzaBuilder;
        delete spicyPizzaBuilder;
        delete hawaiian;
        delete spicy;
		system("PAUSE");
		return EXIT_SUCCESS;
}



